@@ -309,15 +309,19 @@ flatten.dictionary <- function(elms, parent = '', dict = list()) {
 
 #' apply a dictionary or thesarus to an object
 #' 
-#' Convert features into equivalence classes defined by values of a dictionary
-#' object.  
-#' @note Selecting only features defined in a "dictionary" is traditionally
-#' known in text analysis as a dictionary method, even though technically this is more like a thesarus.
-#' If a more truly thesaurus-like application is desired, set \code{keeponly = FALSE} to convert features 
-#' defined as values in a dictionary into their keys, while keeping all other features.
-#' @return an object of the type passed with the value-matching features replaced by dictionary keys
+#' Convert features into equivalence classes defined by values of a dictionary 
+#' object.
+#' @note Selecting only features defined in a "dictionary" is traditionally 
+#'   known in text analysis as a \emph{dictionary method}, even though
+#'   technically this they operate more like a thesarus.  If a more truly thesaurus-like
+#'   application is desired, set \code{keeponly = FALSE} to convert features 
+#'   defined as values in a dictionary into their keys, while keeping all other
+#'   features.
+#' @return an object of the type passed with the value-matching features
+#'   replaced by dictionary keys
 #' @param x object to which dictionary or thesaurus will be supplied
-#' @param dictionary the \link{dictionary}-class object that will be applied to \code{x}
+#' @param dictionary the \link{dictionary}-class object that will be applied to
+#'   \code{x}
 #' @export
 applyDictionary <- function(x, dictionary, ...) {
     UseMethod("applyDictionary")
@@ -325,7 +329,7 @@ applyDictionary <- function(x, dictionary, ...) {
 
 #' @rdname applyDictionary
 #' @param exclusive if \code{TRUE}, remove all features not in dictionary, 
-#'   otherwise, replace values in dictionary keys with keys while leaving other 
+#'   otherwise, replace values in dictionary with keys while leaving other 
 #'   features unaffected
 #' @param valuetype how to interpret dictionary values: \code{"glob"} for 
 #'   "glob"-style wildcard expressions (the format used in Wordstat and LIWC
@@ -365,6 +369,8 @@ applyDictionary.dfm <- function(x, dictionary, exclusive = TRUE, valuetype = c("
                                 verbose = TRUE, ...) {
     valuetype <- match.arg(valuetype)
     dictionary <- flatten.dictionary(dictionary)
+    if (length(addedArgs <- list(...)))
+        warning("Argument", ifelse(length(addedArgs)>1, "s ", " "), names(addedArgs), " not used.", sep = "")
     
     if (verbose) cat("applying a dictionary consisting of ", length(dictionary), " key", 
                      ifelse(length(dictionary) > 1, "s", ""), "\n", sep="")
@@ -379,7 +385,7 @@ applyDictionary.dfm <- function(x, dictionary, exclusive = TRUE, valuetype = c("
     newFeatures <- names(dictionary)
     uniqueFeatures <- features(x)
     newFeatureIndexList <- lapply(dictionary, function(x) {
-        # ind <- grep(paste(x, collapse = "|"), uniqueFeatures, ignore.case = case_insensitive)
+        # ind <- grep(paste(x, collapse = "|"), uniqueFeatures, ignore.case = case_insensitive)
         if (valuetype == "fixed") {
             if (case_insensitive)  
                 ind <- which(toLower(uniqueFeatures) %in% (toLower(x)))
