@@ -34,6 +34,10 @@ encoding <- function(x, verbose = TRUE, ...) {
 #' @export
 encoding.character <- function(x, verbose = TRUE, ...) {
 
+    addedArgs <- names(list(...))
+    if (length(addedArgs) && any(!(addedArgs %in% names(formals(stringi::stri_enc_detect)))))
+        warning("Argument", ifelse(length(addedArgs)>1, "s ", " "), addedArgs, " not used.", sep = "", noBreaks. = TRUE)
+
     confidence <- conf <- NULL
     n <- 1
 
