@@ -155,6 +155,8 @@ setMethod("textfile",
                     cache = "ANY", 
                     docvarsfrom="missing", dvsep="missing", docvarnames="missing"),
           definition = function(file, textField, cache = FALSE, ...) {
+              if (length(addedArgs <- list(...)))
+                  warning("Argument", ifelse(length(addedArgs)>1, "s ", " "), names(addedArgs), " not used.", sep = "")
               if (length(textField) != 1)
                   stop("textField must be a single field name or column number identifying the texts.")
               fileType <- getFileType(file)
@@ -175,6 +177,9 @@ setMethod("textfile",
                     encodingFrom="ANY", encodingTo="ANY", cache = "ANY",
                     docvarsfrom="missing", dvsep="missing", docvarnames="missing"),
           definition = function(file, encodingFrom = NULL, encodingTo = "UTF-8", cache = FALSE, ...) {
+              if (length(addedArgs <- list(...)))
+                  warning("Argument", ifelse(length(addedArgs)>1, "s ", " "), names(addedArgs), " not used.", sep = "")
+              
               fileType <- getFileType(file)
               if (fileType=="filemask") {
                   sources <- get_docs(file, encodingFrom, encodingTo)
@@ -193,6 +198,8 @@ setMethod("textfile",
                     docvarsfrom="character", dvsep="ANY", docvarnames="ANY"),
           definition = function(file, textField=NULL, cache = FALSE, 
                                 docvarsfrom=c("headers"), dvsep="_", docvarnames=NULL, ...) {
+              if (length(addedArgs <- list(...)))
+                  warning("Argument", ifelse(length(addedArgs)>1, "s ", " "), names(addedArgs), " not used.", sep = "")
               fileType <- getFileType(file)
               if (fileType=="filemask") {
                   sources <- get_docs(file, encodingFrom, encodingTo)
@@ -249,6 +256,7 @@ get_docs <- function(filemask, encodingFrom = NULL, encodingTo = "UTF-8") {
     pattern <- getRootFileNames(filemask)
     # get the directory name
     path <- substr(filemask, 1, nchar(filemask) - nchar(pattern))
+    if (path == "") path <- "."
     # get the filenames
     filenames <- list.files(path, pattern, full.names=TRUE)
     # read texts from call to get_doc, discarding any docv
@@ -493,7 +501,9 @@ texts.corpusSource <- function(x, groups = NULL, ...) {
 
 #' @rdname docvars
 #' @export
-docvars.corpusSource <- function(x, ...) {
+docvars.corpusSource <- function(x, field = NULL) {
+    if (!is.null(field))
+        warning("field argument not used for docvars on a corpusSource object", noBreaks. = TRUE)
     x@docvars
 }
 
